package ru.other.company.loy.service.provider;

import javax.annotation.PostConstruct;
import org.slf4j.Logger;
import ru.crystals.pos.api.ext.loyal.dto.LoyaltyResult;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.LoyProviderFeedback;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.PreFiscalizationFeedback;
import ru.crystals.pos.api.plugin.LoyaltyPlugin;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.currency.ExtCurrencyHandler;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.pos.spi.receipt.ReceiptType;

/**
 * Примитивный пример реализации {@link LoyaltyPlugin поставщика "услуг лояльности"}: просто дает скидку в 5% на все позиции в чеке.
 *
 * Внимание! В плагинах лояльности недопустимо манипулировать UI кассы, поэтому вызовы методов класса UIForms здесь запрещены.
 */
@POSPlugin (id = Contstants.NAME)
public class ExampleLoyProvider implements LoyaltyPlugin {

    /**
     * Округлятор денег. Для операций с валютой (округление, вычисление стоимости) следует использовать его.
     */
    @Inject
    ExtCurrencyHandler currencyHandler;

    /**
     * При помощи этого класса осуществляется доступ к настройкам плагина и внешней системы он подчинен.
     * Не требуется создавать экземпляр этого класса, он будет создан загрузчиком плагина автоматически при.
     */
    @Inject
    private IntegrationProperties properties;

    /**
     * Логгер, который будет собирать весь лог плагина. Его не следует создавать вручную, ровно как и использовать
     * какой-либо иной логер.
     */
    @Inject
    Logger log;

    private ServiceProperties serviceProperties;
    private DoDiscountSaleStrategy saleStrategy;
    private PrefiscalizationStrategy prefiscalizationSaleReceiptStrategy;
    private PrefiscalizationStrategy prefiscalizationRefundReceiptStrategy;

    // Вызов метода, аннотированного PostConstruct доступен только для касс версией от 10.2.47.0 и выше.
    @SuppressWarnings("unused")
    @PostConstruct
    void init() {
        serviceProperties = new ServiceProperties(properties);
        StringBuilder sb = new StringBuilder();
        sb.append("URL: ").append(serviceProperties.getProcesingUrl()).append("\n")
                .append("Connection timeout: ").append(serviceProperties.getConnectionTimeout()).append("\n")
                .append("Read timeout: ").append(serviceProperties.getReadTimeout());
        log.info(sb.toString());
        saleStrategy = new DoDiscountSaleStrategy(this);
        prefiscalizationSaleReceiptStrategy = new SaleReceiptPrefiscalizationStrategy(this);
        prefiscalizationRefundReceiptStrategy = new RefundReceiptPrefiscalizationStrategy();
        log.info("Plugin {} loaded", this.getClass());
    }

    @Override
    public LoyProviderFeedback eventReceiptFiscalized(Receipt receipt, LoyaltyResult loyResults) {
        // Наш плагин интересуют только чеки продажи
        if(receipt.getType() != ReceiptType.SALE) {
            return null;
        }
        LoyProviderFeedback feedback = new LoyProviderFeedback();
        feedback.setPayload("Plugin payload goes here");
        return feedback;
    }

    @Override
    public LoyProviderFeedback onSendFeedback(LoyProviderFeedback feedback) {
        log.trace("onSendFeedback({}) called", feedback);
        return null;
    }

    @Override
    public LoyaltyResult doDiscount(Receipt receipt) {
        log.trace("entering process(Receipt). The receipt-no is: {}", receipt == null ? null : receipt.getNumber());
        LoyaltyResult result;
        long stopWatch = System.currentTimeMillis();
        if (receipt == null) {
            log.error("leaving process(Receipt): the argument is NULL");
            return null;
        }
        // Симуляция длительного запроса в процессинг, просто для примера.
        try {
            Thread.sleep(1000L);
        } catch (InterruptedException e) {
            log.error("interrupted!", e);
            return null;
        }
        if(receipt.getType() == ReceiptType.SALE) {
            result = saleStrategy.apply(receipt);
        } else {
            // Не считаем скидки при возврате
            result = new LoyaltyResult();
        }
        log.trace("leaving process(Receipt). The result is: {}; it took {} [ms] to process the request", result, System.currentTimeMillis() - stopWatch);
        return result;
    }


    @Override
    public PreFiscalizationFeedback eventBeforeReceiptFiscalized(Receipt receipt, LoyaltyResult loyResults) {
        if(receipt.getType() == ReceiptType.SALE) {
            return prefiscalizationSaleReceiptStrategy.apply(receipt, loyResults);
        }
        return prefiscalizationRefundReceiptStrategy.apply(receipt, loyResults);
    }
}
