package foo.payment.plugin;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import ru.crystals.pos.api.ext.loyal.dto.DailyReportResult;
import ru.crystals.pos.api.ext.loyal.dto.DailyReportResultCode;
import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraph;
import ru.crystals.pos.api.ext.loyal.dto.SlipParagraphType;
import ru.crystals.pos.api.plugin.PaymentPlugin;
import ru.crystals.pos.api.plugin.RefundPreparationPlugin;
import ru.crystals.pos.api.plugin.TransactionalRefundPlugin;
import ru.crystals.pos.api.plugin.payment.Payment;
import ru.crystals.pos.api.plugin.payment.PaymentResultData;
import ru.crystals.pos.api.plugin.payment.PaymentToRefundReplacement;
import ru.crystals.pos.api.plugin.payment.RefundPreparationResult;
import ru.crystals.pos.api.plugin.payment.TransactionalRefundResult;
import ru.crystals.pos.api.plugin.user.User;
import ru.crystals.pos.api.ui.listener.ConfirmListener;
import ru.crystals.pos.api.ui.listener.DialogListener;
import ru.crystals.pos.api.ui.listener.InputScanNumberFormListener;
import ru.crystals.pos.api.ui.listener.SumToPayFormListener;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.POSInfo;
import ru.crystals.pos.spi.ResBundle;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.equipment.CustomerDisplay;
import ru.crystals.pos.spi.equipment.CustomerDisplayMessage;
import ru.crystals.pos.spi.equipment.SetApiPrinter;
import ru.crystals.pos.spi.equipment.SetApiPrinterException;
import ru.crystals.pos.spi.feedback.Feedback;
import ru.crystals.pos.spi.plugin.payment.CancelRequest;
import ru.crystals.pos.spi.plugin.payment.InvalidPaymentException;
import ru.crystals.pos.spi.plugin.payment.PaymentRequest;
import ru.crystals.pos.spi.plugin.payment.PaymentToRefund;
import ru.crystals.pos.spi.plugin.payment.RefundPreparationRequest;
import ru.crystals.pos.spi.plugin.payment.RefundRequest;
import ru.crystals.pos.spi.plugin.payment.TransactionalRefundRequest;
import ru.crystals.pos.spi.receipt.ProcessedPayment;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.pos.spi.ui.DialogFormParameters;
import ru.crystals.pos.spi.ui.UIForms;
import ru.crystals.pos.spi.ui.payment.SumToPayFormParameters;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static java.util.Collections.singletonList;

@POSPlugin(id = "foo.service.payment")
public class FooPaymentPlugin implements PaymentPlugin, RefundPreparationPlugin, TransactionalRefundPlugin {

    /**
     * Под этим ключом в оплатах будет хранится номер кошелька клиента, по которому
     * выполняется оплата или аннулирование.
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_WALLET_NUMBER = "wallet.number";


    /**
     * Под этим ключом в оплатах будет хранится QR-код, по которому
     * выполняется оплата
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_QR_CODE = "qr.code";


    /**
     * Под этим ключом в оплатах будет храниться код авторизации, получаемый при оплате.
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_AUTHORIZATION_CODE = "authorization.code";

    /**
     * Под этим ключом в оплатах будет храниться идентификатор клиента.
     * @see Payment#getData()
     */
    private static final String PAYMENT_KEY_CLIENT_ID = "client.id";

    /**
     * Под этим ключом в оплатах хранится идентификатор транзакции, который получается при оплате
     * электронным кошельком.
     */
    private static final String PAYMENT_KEY_TRANSACTION_NUMBER = "transaction.number";

    /**
     * Идентификатор оригинальной транзакции чека продажи, которую вернули
     */
    private static final String PAYMENT_KEY_ORIGINAL_TRANSACTION_NUMBER = "original.transaction.number";

    /**
     * Признак, что в этом чеке уже была неуспешная попытка вернуть оплату через плагин
     */
    private static final String PAYMENT_KEY_REFUND_BY_PLUGIN_FAILED = "refund.by.plugin.failed";

    @Inject
    private POSInfo pos; // параметры кассы

    @Inject
    private IntegrationProperties properties; // настройки интеграции и плагина

    @Inject
    private UIForms ui; // формы кассы

    @Inject
    private CustomerDisplay customerDisplay;

    @Inject
    private SetApiPrinter printer;

    @Inject
    private Logger log; // логирование

    @Inject
    private ResBundle res; // локализованные ресурсы

    @PostConstruct
    private void init() {
        log.info("Plugin {} loaded", this.getClass());
    }

    @Override
    public void doPayment(PaymentRequest paymentRequest) {
        inputSum(paymentRequest, paymentRequest.getReceipt().getSurchargeSum());
    }

    private void inputSum(PaymentRequest paymentRequest, BigDecimal defaultSum) {
        // Показываем форму ввода суммы к оплате
        SumToPayFormParameters parameters = new SumToPayFormParameters(res.getString("payment.name"), paymentRequest.getReceipt());
        parameters.setInputHint(res.getString("enter.sum.to.pay"));
        parameters.setDefaultSum(defaultSum);
        ui.getPaymentForms().showSumToPayForm(parameters, new SumToPayFormListener() {

            @Override
            public void eventCanceled() {
                // была нажата <Отмена> завершим процесс
                paymentRequest.getPaymentCallback().paymentNotCompleted();
            }

            @Override
            public void eventSumEntered(BigDecimal sumToPay) {
                if (properties.getServiceProperties().getBoolean("use.qr.on.pos")) {
                    sumEnteredForQr(paymentRequest, sumToPay);
                } else {
                    sumEntered(paymentRequest, sumToPay);
                }
            }
        });
    }

    /**
     * Пример реализации оплаты по QR-коду, показываемому на кассе (на экране покупателя или с печатью на принтере)
     */
    private void sumEnteredForQr(PaymentRequest paymentRequest, BigDecimal sumToPay) {
        boolean canShowQrOnCustomerDisplay = false;
        try {

            // Обращаемся в сервисе оплаты для получения QR-кода оплаты
            ui.showSpinnerForm(res.getString("connecting"));
            // Эмулируем время выполнения запроса
            Thread.sleep(1000);
            String qrPayload = "https://qr.nspk.ru/AD100004BAL7227F9BNP6KNE007J9B3K?type=02&sum=12345&cur=RUB&crc=AB75";

            // Можно ли показать QR на экране покупателя
            canShowQrOnCustomerDisplay = customerDisplay.canShowQr();
            if (canShowQrOnCustomerDisplay) {
                // Если можно показать, то передаем QR, подсказку для покупателя и сумму оплаты
                // (сумма и подсказка могут не отображаться на специализированных QR-дисплеях)
                customerDisplay.display(new CustomerDisplayMessage(res.getString("scanQR"), qrPayload, sumToPay));
            } else {
                // Если показать QR на экране нельзя, печатаем его на предчеке на принтере
                final Slip qrSlip = new Slip();
                qrSlip.getParagraphs().add(new SlipParagraph(SlipParagraphType.TEXT, res.getString("scanQR")));
                qrSlip.getParagraphs().add(new SlipParagraph(SlipParagraphType.QR, qrPayload));
                qrSlip.getParagraphs().add(new SlipParagraph(SlipParagraphType.TEXT, sumToPay.toString()));
                try {
                    printer.print(qrSlip);
                } catch (SetApiPrinterException e) {
                    // Не удалось напечатать предчек - не сможем провести оплату
                    ui.showErrorForm(res.getString("print.qr.error"), () -> paymentRequest.getPaymentCallback().paymentNotCompleted());
                    return;
                }
            }

            // Покупатель сканирует QR в приложении, ожидаем, когда статус оплаты изменится в сервисе оплаты
            ui.showSpinnerForm(res.getString("customer.scanning.qr"));
            // Эмулируем время выполнения запроса
            Thread.sleep(3000);

            if (canShowQrOnCustomerDisplay) {
                // Если показывали сообщение на экране покупателя, скрываем его - больше его сканировать не нужно
                customerDisplay.clear();
            }

            // читаем необходимые настройки плагина:
            String url = properties.getServiceProperties().get("foo.service.url");
            int timeout = properties.getServiceProperties().getInt("timeout", 10000);

            // Можно сохранять какие либо параметры необходимые для работы сервиса
            int persistCounter = properties.getServiceProperties().getInt("PersistedOperationCounter", 0);
            persistCounter++;
            properties.getServiceProperties().setInt("PersistedOperationCounter", persistCounter);

            // читаем необходимые настройки кассы:
            int posNumber = pos.getPOSNumber();
            int shopNumber = pos.getShopNumber();

            // выполнение удаленного вызова, получение ответа
            // формирование данных для callback
            Thread.sleep(2000);
            Payment payment = new Payment();
            payment.setSum(sumToPay);
            payment.getData().put(PAYMENT_KEY_AUTHORIZATION_CODE, "385281233141592");
            payment.getData().put(PAYMENT_KEY_CLIENT_ID, "4815162342");

            // формируем слип для печати
            StringBuilder sb = new StringBuilder();
            sb.append(res.getString("slip.header")).append("\n")
                    .append(sumToPay.toString())
                    .append(res.getString("slip.rub")).append("\n")
                    .append(res.getString("slip.transaction.number"))
                    .append("\n")
                    .append("01234567890");
            payment.getSlips().add(sb.toString());

            // Добавим в мапу реквизитов номер кошелька, которым выполнялась оплата.
            // Таким образом, в случае аннулирования чека, можно осуществить возврат средств
            // на этот же кошелек, без необходимости повторно запрашивать номер.
            payment.getData().put(PAYMENT_KEY_QR_CODE, qrPayload);
            // Также сохраним идентификатор транзакции платежа
            payment.getData().put(PAYMENT_KEY_TRANSACTION_NUMBER, "01234567890");

            // процесс оплаты успешно завершен
            paymentRequest.getPaymentCallback().paymentCompleted(payment);
            // Касса продолжает процесс оплаты и регистрации чека.
            // С этого момента отображение форм уже недопустимо.
            // Попытка отобразить форму после отправка callback приведет к исключению
            // IncorrectStateException
        } catch (Exception e) {
            if (canShowQrOnCustomerDisplay) {
                // в случае ошибки на оплате скрываем QR, если показывали его
                customerDisplay.clear();
            }
            ui.showErrorForm(res.getString("connection.error"), new ConfirmListener() {
                @Override
                public void eventConfirmed() {
                    paymentRequest.getPaymentCallback().paymentNotCompleted();
                }
            });
        }
    }

    private void sumEntered(PaymentRequest paymentRequest, BigDecimal sumToPay) {
        ui.getInputForms().showInputScanNumberForm(res.getString("payment.name"), res.getString("scanQR"), res.getString("wallet.number"), 16, new InputScanNumberFormListener() {

            @Override
            public void eventBarcodeScanned(String barcode) {
                remoteRequest(paymentRequest, barcode, sumToPay);
            }

            @Override
            public void eventInputComplete(String number) {
                remoteRequest(paymentRequest, number, sumToPay);
            }

            @Override
            public void eventCanceled() {
                // если нажали отмену, возвращаемся в форме ввода суммы к оплате,
                // на форме будет отображена введенная сумма
                inputSum(paymentRequest, sumToPay);
            }
        });
    }


    private void remoteRequest(PaymentRequest paymentRequest, String walletNumber, BigDecimal sumToPay) {
        ui.showSpinnerForm(res.getString("connecting"));
        // читаем необходимые настройки плагина:
        String url = properties.getServiceProperties().get("foo.service.url");
        int timeout = properties.getServiceProperties().getInt("timeout", 10000);

        // Можно сохранять какие либо параметры необходимые для работы сервиса
        int persistCounter = properties.getServiceProperties().getInt("PersistedOperationCounter", 0);
        persistCounter++;
        properties.getServiceProperties().setInt("PersistedOperationCounter", persistCounter);

        // читаем необходимые настройки кассы:
        int posNumber = pos.getPOSNumber();
        int shopNumber = pos.getShopNumber();

        try {
            // выполнение удаленного вызова, получение ответа
            // формирование данных для callback
            Thread.sleep(2000);
            Payment payment = new Payment();
            payment.setSum(sumToPay);
            payment.getData().put(PAYMENT_KEY_AUTHORIZATION_CODE, "385281233141592");
            payment.getData().put(PAYMENT_KEY_CLIENT_ID, "4815162342");

            // формируем слип для печати
            StringBuilder sb = new StringBuilder();
            sb.append(res.getString("slip.header")).append("\n")
                .append(sumToPay.toString())
                .append(res.getString("slip.rub")).append("\n")
                .append(res.getString("slip.transaction.number"))
                .append("\n")
                .append("01234567890");
            payment.getSlips().add(sb.toString());

            // Добавим в мапу реквизитов номер кошелька, которым выполнялась оплата.
            // Таким образом, в случае аннулирования чека, можно осуществить возврат средств
            // на этот же кошелек, без необходимости повторно запрашивать номер.
            payment.getData().put(PAYMENT_KEY_WALLET_NUMBER, walletNumber);
            // Также сохраним идентификатор транзакции платежа
            payment.getData().put(PAYMENT_KEY_TRANSACTION_NUMBER, "01234567890");

            // процесс оплаты успешно завершен
            paymentRequest.getPaymentCallback().paymentCompleted(payment);
            // Касса продолжает процесс оплаты и регистрации чека.
            // С этого момента отображение форм уже недопустимо.
            // Попытка отобразить форму после отправка callback приведет к исключению
            // IncorrectStateException
        } catch (Exception e) {
            ui.showErrorForm(res.getString("connection.error"), new ConfirmListener() {
                @Override
                public void eventConfirmed() {
                    paymentRequest.getPaymentCallback().paymentNotCompleted();
                }
            });
        }
    }

    @Override
    public void doPaymentCancel(CancelRequest cancelRequest) {
        ui.showSpinnerForm(res.getString("cancellation.in.progress"));
        // Получим обратно номер кошелька, который мы запросили при оплате и с которого же была оплата произведена, и другие реквизиты,
        // которые бы могли понадобиться.
        String walletNumber = cancelRequest.getPayment().getData().get(PAYMENT_KEY_WALLET_NUMBER);
        String transactionId = cancelRequest.getPayment().getData().get(PAYMENT_KEY_TRANSACTION_NUMBER);
        BigDecimal sum = cancelRequest.getPayment().getSum();

        try {
            Payment cancelledPayment = cancelRequest.getPayment();
            // Эмулируем длительную операцию аннулирования оплаты.
            Thread.sleep(2000);
            // Просто чтобы дать возможность посмотреть диалоговое окно об ошибке при аннулировании
            if(StringUtils.isBlank(walletNumber)) {
                throw new IllegalArgumentException("Wallet number cannot be empty!");
            }
            cancelledPayment.getSlips().add("Cancellation slip goes here");
            // Аннулирование прошло успешно, уведомляем кассу об этом.
            // Изменять сумму в чеке нет нужды - касса сама обрабатывает результат аннулирования.
            cancelRequest.getPaymentCallback().paymentCompleted(cancelledPayment);
        } catch (Exception ex) {
            log.error("An error has occurred during check cancellation", ex);
            // Аннулирование не удалось, предложим пользователю попробовать провести аннулирование ещё раз или
            // продолжить без подтверждения об аннулирвоании.
            DialogFormParameters dialogModel = new DialogFormParameters(
                    res.getString("cancellation.failed.dialog"),
                    res.getString("cancellation.failed.dialog.button.proceed"),
                    res.getString("cancellation.failed.dialog.button.retry")
            );
            ui.showDialogForm(dialogModel, new DialogListener() {
                @Override
                public void eventButton1pressed() {
                    // Выбран вариант аннулирования чека
                    try {
                        cancelRequest.getPaymentCallback().paymentCompleted(cancelRequest.getPayment());
                    } catch (InvalidPaymentException pex) {
                        log.error("Error during check cancellation", pex);
                        ui.showErrorForm(res.getString("cancellation.generic.error"), () -> cancelRequest.getPaymentCallback().paymentNotCompleted());
                    }
                }

                @Override
                public void eventButton2pressed() {
                    // Выбран вариант повтора попытки аннулирования
                    doPaymentCancel(cancelRequest);
                }

                @Override
                public void eventCanceled() {
                    // Пользователь может отказаться от аннулирования чека вообще.
                    cancelRequest.getPaymentCallback().paymentNotCompleted();
                }
            });
        }
    }

    @Override
    public void doRefund(RefundRequest refundRequest) {
        // Показываем форму ввода суммы к оплате
        SumToPayFormParameters parameters = new SumToPayFormParameters(res.getString("refund.name"), refundRequest.getRefundReceipt());
        parameters.setInputHint(res.getString("enter.sum.to.refund"));
        parameters.setDefaultSum(refundRequest.getSumToRefund());
        ui.getPaymentForms().showSumToPayForm(parameters, new SumToPayFormListener() {

            @Override
            public void eventCanceled() {
                // была нажата <Отмена> завершим процесс
                refundRequest.getPaymentCallback().paymentNotCompleted();
            }

            @Override
            public void eventSumEntered(BigDecimal sumToPay) {
                sumRefundEntered(refundRequest, sumToPay);
            }

			private void sumRefundEntered(RefundRequest refundRequest, BigDecimal sumToPay) {
		        ui.showSpinnerForm(res.getString("connecting"));
		        // читаем необходимые настройки плагина:
		        String url = properties.getServiceProperties().get("foo.service.url");
		        int timeout = properties.getServiceProperties().getInt("timeout", 10000);

		        // читаем необходимые настройки кассы:
		        int posNumber = pos.getPOSNumber();
		        int shopNumber = pos.getShopNumber();

                // Получить информацию о кассире
                User currentCashier = pos.getUser();

		        try {
		            // выполнение удаленного вызова, получение ответа
		            // формирование данных для callback
		        	Thread.sleep(2000);
		            Payment payment = new Payment();
		            payment.setSum(sumToPay);
		            payment.getData().put(PAYMENT_KEY_AUTHORIZATION_CODE, "some code");
		            payment.getData().put("client.id", "some client ID");

		            // формируем слип для печати
		            StringBuilder sb = new StringBuilder();
		            sb.append(res.getString("refund.slip.header")).append("\n")
		                .append(sumToPay.toString())
		                .append(res.getString("slip.rub")).append("\n")
		                .append(res.getString("slip.transaction.number")).append("\n")
		                .append("01234567890");
		            payment.getSlips().add(sb.toString());

		            // процесс возврата успешно завершен
		            try {
                        refundRequest.getPaymentCallback().paymentCompleted(payment);
                    } catch (InvalidPaymentException e) {
                        log.error("Failed to commit payment", e);
                    }
		            // Касса продолжает процесс возврата и регистрации чека.
		            // С этого момента отображение форм уже недопустимо.
		            // Попытка отобразить форму после отправка callback приведет к исключению
		            // IncorrectStateException
		        } catch (Exception e) {
		            ui.showErrorForm(res.getString("connection.error"), new ConfirmListener() {
		                @Override
		                public void eventConfirmed() {
		                	refundRequest.getPaymentCallback().paymentNotCompleted();
		                }
		            });
		        }
			}
        });
    }

    @Override
    public boolean isAvailable() {
    	// если необходимые для работы настройки не заполнены
        return properties.getServiceProperties().get("foo.service.url") != null;
    }

    @Override
    public DailyReportResult createDailyReport() {
        DailyReportResult result = new DailyReportResult();
        // Просто для примера и солидности затянем выполнение на некоторое время.
        try {
            Thread.sleep(2000);
        } catch (InterruptedException iex) {
            log.error("Suddenly awoken.");
        }
        result.getPrintStrings().add(res.getString("service.name"));
        result.getPrintStrings().add("ПРОДАЖ: 100");
        result.getPrintStrings().add("ВОЗВРАТОВ: 0");
        result.getPrintStrings().add("ПРОДАЖА: 10000 руб.");
        result.getPrintStrings().add("ВОЗВРАТ: 0 руб.");
        result.getPrintStrings().add("ОБОРОТ: 10000 руб.");
        result.setResultCode(DailyReportResultCode.SUCCESS);
        return result;
    }

    @Override
    public boolean hasDailyReports() {
        // Этот плагин умеет проводить сверку итогов, потому что мы реализовали метод createDailyReport,
        // поэтому, чтобы сверка по этому плагину была доступна, следует рассказать кассе, что метод можно вызывать.
        // Для этого мы возвращаем true здесь.
        return true;
    }

    @Override
    public Feedback eventReceiptFiscalized(Receipt receipt, boolean isCancelReceipt) {
        log.trace("Receipt fiscalized event. Receipt creation date: {}", receipt.getDateCreate());
        Feedback feedback = null;
        if (receipt.getPayments().stream().map(ProcessedPayment::getPluginId).anyMatch("foo.service.payment"::equals)) {
            feedback = new Feedback("Some payload for sending to server later. IsCancel=" + isCancelReceipt);
        }
        return feedback;
    }

    @Override
    public void onRepeatSend(Feedback feedback) throws Exception {
        log.trace("Entering onRepeatSend goodsFeedback.payload={}", feedback.getPayload());
        log.trace("Leaving onRepeatSend");
    }

    @Override
    public RefundPreparationResult prepareRefund(RefundPreparationRequest request) {
        // Предположим, при возврате по чеку мы хотим
        // - при определенных условиях (например, при ошибке возврата) разрешить возврат оплат нашим плагином другими способами
        // - объединить несколько оплат и чека продажи (например, подарочными картами) для возврата одной транзакцией (например, на одну карту)
        // - изменить название типа оплаты в списке транзакций для возврата, например, чтобы различать несколько проведенных оплат между собой

        // Проверим, что в этом чеке возврата еще не было неудачных попыток возврата через плагин.
        // Смотрим на дополнительный атрибут чека, который мы сохранили при неудачной попытке возврата в PaymentPlugin.doRefund
        // или TransactionalRefundPlugin.doTransactionalRefund (если плагин реализует соответствующий интерфейс)
        final boolean refundByPluginFailed = Boolean.parseBoolean(request.getRefundReceipt().getData().get(PAYMENT_KEY_REFUND_BY_PLUGIN_FAILED));
        if (refundByPluginFailed) {
            final boolean disableRefundPluginAfterFirstFailure = properties.getPluginProperties().getBoolean("disable.refund.by.plugin.after.first.failure", false);
            final boolean allowExtraPaymentTypesToRefundOnFailure = properties.getPluginProperties().getBoolean("allow.extra.payments.types.to.refund.on.failure", false);

            // Если настройки плагина запрещают повторную попытку возврата через плагин при ошибке
            if (disableRefundPluginAfterFirstFailure) {
                // то делаем одну общую замену для всех оплат плагина
                // (в случае запрета возврата через плагин нет разницы - отдельно для каждой или для всех сразу)
                final PaymentToRefundReplacement allReplacement = PaymentToRefundReplacement.forPayments(request.getPaymentsToRefund());
                // и установим запрет возврата через плагин
                allReplacement.setRestrictRefundByPlugin(true);
                // Опционально разрешим возвращать дополнительными способами, которые обычно недоступны
                // (список возможных способов задаются общей настройкой "Способы возврата, дополнительно доступные по решению плагина")
                allReplacement.setUseAlternativePaymentTypesToRefund(allowExtraPaymentTypesToRefundOnFailure);
                return new RefundPreparationResult(singletonList(allReplacement));
            }

            // Если после ошибки возврата плагином можно снова пытаться сделать возврат, то подготавливаем обычный результат
            final RefundPreparationResult result = makePreparationResult(request);
            // и при необходимости активируем дополнительные способы ("Способы возврата, дополнительно доступные по решению плагина")
            result.getPaymentReplacements().forEach(replacement -> replacement.setUseAlternativePaymentTypesToRefund(allowExtraPaymentTypesToRefundOnFailure));
            return result;
        }
        // Если ошибок при возврате не было, то подготавливаем обычный результат
        return makePreparationResult(request);
    }

    private RefundPreparationResult makePreparationResult(RefundPreparationRequest request) {
        final boolean allowRefundMultipleTransactions = properties.getPluginProperties().getBoolean("allow.refund.multiple.transactions", false);
        // Если в настройках плагина включена возможность возврата нескольких транзакций оплаты одной транзакцией
        // и для возврата доступно несколько транзакций из исходного чека (например, несколько подарочных карт вернуть на одну), то сгруппируем оплаты в одну
        if (allowRefundMultipleTransactions && request.getPaymentsToRefund().size() > 1) {
            return new RefundPreparationResult(singletonList(PaymentToRefundReplacement.forPayments(request.getPaymentsToRefund())));
        }
        // Если возврат возможен только по одной транзакции, то сформируем для каждой исходной оплаты замену
        return new RefundPreparationResult(request.getPaymentsToRefund().stream()
                .map(toRefund -> {
                    final String walletNumber = toRefund.getOriginalPayment().getData().get(PAYMENT_KEY_WALLET_NUMBER);
                    // Изменим название типа оплаты в списке транзакций для возврата, добавив номер кошелька
                    return PaymentToRefundReplacement.withDescription(toRefund, String.format(res.getString("refund.name.with.number"), walletNumber));
                })
                .collect(Collectors.toList()));
    }

    @Override
    public void doTransactionalRefund(TransactionalRefundRequest refundRequest) {
        final List<PaymentToRefund> paymentsToRefund = refundRequest.getPaymentsToRefund();
        final Map<String, String> resultPaymentData = new HashMap<>();
        if (paymentsToRefund.size() > 1) {

            for (PaymentToRefund paymentToRefund : paymentsToRefund) {
                final String transactionId = paymentToRefund.getOriginalPayment().getData().get(PAYMENT_KEY_TRANSACTION_NUMBER);
                final BigDecimal transactionSumToRefund = paymentToRefund.getSumToRefund();
                // (Проверим в процессинге, что переданные транзакции доступны к возврату на указанные суммы)
            }
            resultPaymentData.put(PAYMENT_KEY_ORIGINAL_TRANSACTION_NUMBER, paymentsToRefund.stream()
                    .map(PaymentToRefund::getOriginalPayment)
                    .map(op -> op.getData().get(PAYMENT_KEY_TRANSACTION_NUMBER))
                    .collect(Collectors.joining(",")));

            final BigDecimal totalSumToRefund = refundRequest.getSumToRefund();
            // (Выполним возврат в процессинге одной транзакцией на общую сумму всех транзакций)
        } else {
            //
            final PaymentToRefund paymentToRefund = paymentsToRefund.get(0);
            final String transactionId = paymentToRefund.getOriginalPayment().getData().get(PAYMENT_KEY_TRANSACTION_NUMBER);
            final BigDecimal transactionSumToRefund = paymentToRefund.getSumToRefund();
            // (Выполним возврат в процессинге отдельной транзакции)

            resultPaymentData.put(PAYMENT_KEY_ORIGINAL_TRANSACTION_NUMBER, transactionId);
        }

        // Для эмуляции ответа процессинга покажем диалог
        ui.showDialogForm(new DialogFormParameters(
                res.getString("transactional.refund.dialog"),
                res.getString("transactional.refund.dialog.button.yes"),
                res.getString("transactional.refund.dialog.button.no")
        ), new DialogListener() {
            @Override
            public void eventButton1pressed() {

                final PaymentResultData resultData = new PaymentResultData();

                resultData.getData().putAll(resultPaymentData);
                resultData.getData().put(PAYMENT_KEY_TRANSACTION_NUMBER, "4145763312");

                // формируем слип для печати
                StringBuilder sb = new StringBuilder();
                sb.append(res.getString("refund.slip.header")).append("\n")
                        .append(refundRequest.getSumToRefund().toString())
                        .append(res.getString("slip.rub")).append("\n")
                        .append(res.getString("slip.transaction.number")).append("\n")
                        .append("01234567890");
                resultData.getSlips().add(sb.toString());

                refundRequest.getOperationCallback().refundCompleted(new TransactionalRefundResult(resultData));
            }

            @Override
            public void eventButton2pressed() {
                final PaymentResultData resultData = new PaymentResultData();
                resultData.getExtendedReceiptAttributes().put(PAYMENT_KEY_REFUND_BY_PLUGIN_FAILED, Boolean.TRUE.toString());
                refundRequest.getOperationCallback().operationNotCompleted(new TransactionalRefundResult(resultData));
            }

            @Override
            public void eventCanceled() {
                refundRequest.getOperationCallback().operationNotCompleted(null);
            }
        });

    }
}
