package ru.crystals.setapi.plugins.test.webview.plugin;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import ru.crystals.pos.api.plugin.PaymentPlugin;
import ru.crystals.pos.api.plugin.payment.Payment;
import ru.crystals.pos.api.ui.listener.ScanFormListener;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.plugin.payment.CancelRequest;
import ru.crystals.pos.spi.plugin.payment.InvalidPaymentException;
import ru.crystals.pos.spi.plugin.payment.PaymentRequest;
import ru.crystals.pos.spi.plugin.payment.RefundRequest;
import ru.crystals.pos.spi.plugin.ui.WebViewCallback;
import ru.crystals.pos.spi.ui.input.UIInputForms;
import ru.crystals.pos.spi.ui.webview.EventsProcessingRule;
import ru.crystals.pos.spi.ui.webview.ShowWebViewParameters;
import ru.crystals.pos.spi.ui.webview.UIWebViewForms;
import ru.crystals.pos.spi.ui.webview.WebViewNotAvailableException;
import ru.crystals.pos.spi.ui.webview.WebViewShowType;
import ru.crystals.setapi.plugins.test.webview.plugin.dto.WebViewMessageDTO;
import ru.crystals.setapi.plugins.test.webview.plugin.dto.WebViewResponseDTO;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.EnumSet;

@POSPlugin(id = "webview.test.plugin.payment")
public class WebViewPaymentPlugin implements PaymentPlugin {
    private static final String WEBVIEW_URL = "http://127.0.0.1:8000/test.html?sum_to_pay=%d&timestamp=%d";

    private static final String IGNORE_SCANNER_EVENTS_KEY = "webview.test.plugin.ignore.scanner";

    @Inject
    private UIWebViewForms webViewForms;

    @Inject
    private UIInputForms inputForms;

    @Inject
    private IntegrationProperties integrationProperties;

    @Inject
    private Logger log;

    private final ObjectMapper objectMapper = new ObjectMapper();

    private volatile WebViewCallback callback;

    @Override
    public void doPayment(PaymentRequest paymentRequest) {
        long sumToPay = paymentRequest.getReceipt().getSurchargeSum().movePointRight(2).longValue();
        String url = String.format(WEBVIEW_URL, sumToPay, System.currentTimeMillis());

        ShowWebViewParameters parameters = ShowWebViewParameters.builder()
                .url(url)
                .showType(WebViewShowType.FULL_SCREEN)
                .eventProcessingRules(getEventProcessingRules())
                .messageListener(((message, callback) -> processMessage(message, callback, paymentRequest)))
                .scanFormListener(getScanFormListener(paymentRequest))
                .closeListener(() -> paymentRequest.getPaymentCallback().paymentNotCompleted())
                .build();

        try {
            callback = webViewForms.showWebView(parameters);
        } catch (WebViewNotAvailableException e) {
            log.error("Can't open WebView form", e);
            paymentRequest.getPaymentCallback().paymentNotCompleted();
        }
    }

    @Override
    public void doPaymentCancel(CancelRequest cancelRequest) {
        try {
            cancelRequest.getPaymentCallback().paymentCompleted(cancelRequest.getPayment());
        } catch (InvalidPaymentException e) {
            log.error("Can't cancel payment", e);
        }
    }

    @Override
    public void doRefund(RefundRequest refundRequest) {
        Payment payment = new Payment();
        payment.setSum(refundRequest.getSumToRefund());
        try {
            refundRequest.getPaymentCallback().paymentCompleted(payment);
        } catch (InvalidPaymentException e) {
            log.error("Can't do refund", e);
        }
    }

    @Override
    public boolean isAvailable() {
        return true;
    }

    private void processBarcode(String barcode, PaymentRequest request) {
        WebViewResponseDTO responseDTO = new WebViewResponseDTO(WebViewResponseDTO.ResponseType.BARCODE_SCANNED, barcode);
        try {
            String responseJson = objectMapper.writeValueAsString(responseDTO);
            callback.send(responseJson);
        } catch (JsonProcessingException e) {
            log.error("Can't parse message from web view", e);
            callback.close();
            request.getPaymentCallback().paymentNotCompleted();
        }
    }

    private void processMessage(String message, WebViewCallback callback, PaymentRequest request) {
        log.error("Received message from WebView: {}", message);
        try {
            processPayment(message, callback, request);
        } catch (IOException e) {
            log.error("Can't parse message from web view", e);
            callback.close();
            request.getPaymentCallback().paymentNotCompleted();
        } catch (InvalidPaymentException e) {
            log.error("Can't process payment", e);
            request.getPaymentCallback().paymentNotCompleted();
        }
    }

    private void processPayment(String message, WebViewCallback callback, PaymentRequest request) throws IOException, InvalidPaymentException {
        WebViewMessageDTO webViewMessageDTO = objectMapper.readValue(message, WebViewMessageDTO.class);
        String sumEnd = webViewMessageDTO.getHeader().getSumEnd();
        if (sumEnd == null) {
            callback.close();
            request.getPaymentCallback().paymentNotCompleted();
        } else {
            Payment payment = new Payment();
            long sum = Long.parseLong(sumEnd);
            payment.setSum(BigDecimal.valueOf(sum, 2));
            callback.close();
            request.getPaymentCallback().paymentCompleted(payment);
        }
    }

    private EnumSet<EventsProcessingRule> getEventProcessingRules() {
        if (integrationProperties.getServiceProperties().getBoolean(IGNORE_SCANNER_EVENTS_KEY, false)) {
            return EnumSet.of(EventsProcessingRule.IGNORE_SCANNER_EVENTS);
        } else {
            return EnumSet.noneOf(EventsProcessingRule.class);
        }
    }

    private ScanFormListener getScanFormListener(PaymentRequest request) {
        return new ScanFormListener() {
            @Override
            public void eventBarcodeScanned(String barcode) {
                processBarcode(barcode, request);
            }

            @Override
            public void eventCanceled() {
                //nothing
            }
        };
    }
}
