package ru.crystals.pos.api.ext.loyal.dto;

import java.math.BigDecimal;

/**
 * Описывает предоставленную на позицию скидку.
 * <p>
 * NOTE: все денежные суммы представлены в {@link BigDecimal} с двумя знаками после запятой, т.е. scale=2.
 */
public class Discount {

    /**
     * Номер позиции, на которую дана эта скидка.
     */
    private int posNo;

    /**
     * Количество товара в позиции, на которое предоставлена данная скидка.
     * Опциональный параметр
     */
    private BigDecimal quantity;

    /**
     * Номер карты, применение которой спровоцировало срабатывание данной скидки.
     */
    private String cardNumber;

    /**
     * Величина скидки.
     */
    private BigDecimal value;

    /**
     * Идентификатор данной скидки (код скидки).
     */
    private String discountId;

    /**
     * Флаг-признак, указывающий была ли данная скидка скидкой на весь чек (до "размазывания"/распределения данной скидки по позициям).
     * @since 0.0.27
     */
    private boolean receiptWideDiscount = false;

    /**
     * Тип скидки
     * @since 0.0.60
     */
    private DiscountType discountType = DiscountType.SUM;

    /**
     * Наименование акции внешней системы.
     * Опциональный параметр.
     */
    private String discountName;

    /**
     * Признак запрета перераспределения скидки тюнером ФЗ-54 между позициями чека.
     * <p>
     * Необходим в том случае, если размер скидки запрещено менять для избежания превышения заданного порога.
     * <p>
     * NOTE: учитывается только для скидки бонусами {@link DiscountType#BONUSES}.
     */
    private boolean disableRedistribution = false;

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link Discount}.
     * @param posNo Номер позиции, на которую дана эта скидка.
     * @param cardNumber Номер карты, применение которой спровоцировало срабатывание данной скидки.
     * @param value Размер скидки.
     * @param discountId Идентификатор данной скидки (код скидки).
     */
    public Discount(int posNo, String cardNumber, BigDecimal value, String discountId) {
        this.posNo = posNo;
        this.cardNumber = cardNumber;
        this.value = value;
        this.discountId = discountId;
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link Discount}.
     *
     * @param posNo      Номер позиции, на которую дана эта скидка.
     * @param cardNumber Номер карты, применение которой спровоцировало срабатывание данной скидки.
     * @param value      Размер скидки.
     * @param discountId Идентификатор данной скидки (код скидки).
     * @param quantity   Кол-во товара в позиции, на которое предоставлена скидка
     */
    public Discount(int posNo, String cardNumber, BigDecimal value, String discountId, BigDecimal quantity) {
        this.posNo = posNo;
        this.cardNumber = cardNumber;
        this.value = value;
        this.discountId = discountId;
        this.quantity = quantity;
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link Discount}.
     *
     * @param posNo        Номер позиции, на которую дана эта скидка.
     * @param cardNumber   Номер карты, применение которой спровоцировало срабатывание данной скидки.
     * @param value        Размер скидки.
     * @param discountId   Идентификатор данной скидки (код скидки).
     * @param discountName Наименование акции внешней системы.
     */
    public Discount(int posNo, String cardNumber, BigDecimal value, String discountId, String discountName) {
        this.posNo = posNo;
        this.cardNumber = cardNumber;
        this.value = value;
        this.discountId = discountId;
        this.discountName = discountName;
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link Discount}.
     *
     * @param posNo        Номер позиции, на которую дана эта скидка.
     * @param cardNumber   Номер карты, применение которой спровоцировало срабатывание данной скидки.
     * @param value        Размер скидки.
     * @param discountId   Идентификатор данной скидки (код скидки).
     * @param discountName Наименование акции внешней системы.
     * @param quantity     Кол-во товара в позиции, на которое предоставлена скидка.
     */
    public Discount(int posNo, String cardNumber, BigDecimal value, String discountId, String discountName, BigDecimal quantity) {
        this.posNo = posNo;
        this.cardNumber = cardNumber;
        this.value = value;
        this.discountId = discountId;
        this.discountName = discountName;
        this.quantity = quantity;
    }

    /**
     * Вернет номер позиции, на которую дана эта скидка.
     * @return номер позиции.
     */
    public int getPosNo() {
        return posNo;
    }

    /**
     * Вернет номер карты, применение которой спровоцировало срабатывание данной скидки.
     * @return номер карты
     */
    public String getCardNumber() {
        return cardNumber;
    }

    /**
     * Возвращает величину скидки.
     * @return величина скидки.
     */
    public BigDecimal getValue() {
        return value;
    }

    /**
     * Вернет Идентификатор данной скидки (код скидки).
     * @return идентификатор.
     */
    public String getDiscountId() {
        return discountId;
    }

    /**
     * Вернет признак была ли эта скидка дана (в процессинге) на весь чек, а не конкретно на эту позицию.
     * @return {@code false}, если эта скидка была изначально (в процессинге) дана именно на эту позицию
     */
    public boolean isReceiptWideDiscount() {
        return receiptWideDiscount;
    }

    /**
     * Установит признак: "дана ли была эта скидка на весь чек".
     *
     * @param receiptWideDiscount значение, в которое надо установить флаг-признак: "является ли скидка на данную
     *                            позицию частью скидки на чек"
     */
    public void setReceiptWideDiscount(boolean receiptWideDiscount) {
        this.receiptWideDiscount = receiptWideDiscount;
    }

    /**
     * Вернет количество товара в позиции, на которое предоставлена скидка,
     * или <code>null</code>, если скидка предоставлена на всю позицию.
     */
    public BigDecimal getQuantity() {
        return quantity;
    }

    public DiscountType getDiscountType() {
        return discountType;
    }

    public void setDiscountType(DiscountType discountType) {
        this.discountType = discountType;
    }

    /**
     * Вернет наименование акции внешней системы.
     * @return наименование акции внешней системы.
     */
    public String getDiscountName() {
        return discountName;
    }

    public boolean isDisableRedistribution() {
        return disableRedistribution;
    }

    public void setDisableRedistribution(boolean disableRedistribution) {
        this.disableRedistribution = disableRedistribution;
    }

    @Override
    public String toString() {
        return "Discount{" +
                "posNo=" + posNo +
                ", cardNumber='" + cardNumber + '\'' +
                ", value=" + value +
                ", discountId='" + discountId + '\'' +
                ", receiptWideDiscount=" + receiptWideDiscount +
                ", quantity=" + quantity +
                ", discountType=" + discountType +
                ", discountName='" + discountName + '\'' +
                ", disableRedistribution='" + disableRedistribution + '\'' +
                '}';
    }
}
