package ru.crystals.pos.api.ext.loyal.dto;

import java.util.ArrayList;
import java.util.List;

/**
 * Печатный документ (слип), который плагин может добавить к чеку.
 * Слип состоит из набора параграфов ({@link SlipParagraph}, которые могут быть текстовыми или баркодными.
 */
public class Slip {
    /**
     * Порядковый номер слипа в кассовом чеке.
     * Если указан, то слип должен быть добавлен в чек соответствующим по порядку.
     * @since 0.0.46
     */
    private Integer index;

    /**
     * Список строк, составляющих слип. Это не обязательно текстовые строки, могут
     * быть также изображением, баркодом, QR, etc.
     */
    private List<SlipParagraph> paragraphs;

    /**
     * Флаг, определяющий, следует ли слип печатать отдельно или же в составе чека.
     * true если следует печатать отдельно и false если следует включить в состав чека.
     * @since 0.0.41
     */
    private boolean separated = true;

    /**
     * Флаг, определяющий, следует ли на время печати слипа отключить логотип.<br>
     * Логотип это картинка в начале каждого слипа, которая может быть настроена печататься в фискальном регистраторе.
     * true если логотип следует отключить на время печати слипа и false в противном случае.
     * Установка этого флага влияет только на текущий слип и не влияет на слипы последующие.
     * @since 0.0.41
     */
    private boolean disableLogo = false;

    /**
     * Флаг, определяющий, следует ли на время печати слипа отключить реквизиты (шапку и подвал).<br>
     * true если реквизиты следует отключить на время печати слипа и false в противном случае.
     * Установка этого флага влияет только на текущий слип и не влияет на слипы последующие.
     * @since 0.0.46
     */
    private boolean disableRequisites;

    /**
     * Определяет, следует ли отключать отрезку данного документа от предыдущего.<br>
     * true если следует отключить.
     * @since 0.0.46
     */
    private boolean disableCut;

    /**
     * Признак необходимости печати слипа, даже если печать чека, к которому он привязан, отключена.
     * true - если печать слипа необходимо выполнять в любом случае.
     * @since 0.0.74
     */
    private boolean forcePrinting;

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link Slip}.
     */
    public Slip() {
        // Empty.
    }

    /**
     * Возврашает список параграфов, из которых состоит данный слип.
     * @return список параграфов, из которых состоит данный слип или пустой список, если слип пустой.
     */
    public List<SlipParagraph> getParagraphs() {
        if(paragraphs == null) {
            paragraphs = new ArrayList<>();
        }
        return paragraphs;
    }

    /**
     * Возвращает флаг, определяющий, следует ли печатать слип отдельно или в составе чека.
     * @return true если следует печатать отдельно и false если следует печатать слип в составе чека.
     * @since 0.0.41
     */
    public boolean isSeparated() {
        return separated;
    }

    /**
     * Устанавливает флаг, определяющий, следует ли печатать слип отдельно или в составе чека.
     * @param separated true если следует печатать отдельно и false если слип следует печатать в составе чека.
     * @since 0.0.41
     */
    public void setSeparated(boolean separated) {
        this.separated = separated;
    }

    /**
     * Возврвщает флаг, определяющий, следует ли отключить печать логотипа на время печати данного слипа.
     * @return true если следует и false в противном случае.
     * @since 0.0.41
     */
    public boolean isDisableLogo() {
        return disableLogo;
    }

    /**
     * Устанавливает флаг, определяющий, следует ли отключить печать логотипа на время печати данного слипа.
     * @param disableLogo true если следует и false в противном случае.
     * @since 0.0.41
     */
    public void setDisableLogo(boolean disableLogo) {
        this.disableLogo = disableLogo;
    }

    public boolean isDisableRequisites() {
        return disableRequisites;
    }

    public void setDisableRequisites(boolean disableRequisites) {
        this.disableRequisites = disableRequisites;
    }

    public boolean isDisableCut() {
        return disableCut;
    }

    public void setDisableCut(boolean disableCut) {
        this.disableCut = disableCut;
    }

    public boolean isForcePrinting() {
        return forcePrinting;
    }

    public void setForcePrinting(boolean forcePrinting) {
        this.forcePrinting = forcePrinting;
    }

    public Integer getIndex() {
        return index;
    }

    public void setIndex(Integer index) {
        this.index = index;
    }

    @Override
    public String toString() {
        return "Slip{" +
                "index=" + index +
                ", paragraphs=" + paragraphs +
                ", separated=" + separated +
                ", disableLogo=" + disableLogo +
                ", disableRequisites=" + disableRequisites +
                ", disableCut=" + disableCut +
                ", forcePrinting=" + forcePrinting +
                '}';
    }
}
