package ru.crystals.pos.api.ext.loyal.dto.auxiliaries;

import ru.crystals.pos.spi.feedback.Feedback;

/**
 * Некий фидбэк, который плагину лояльности необходимо отправить в свой процессинг (ПЦ - Процессинговый Центр) по результатам расчета/применения скидок в чеке.
 */
public class LoyProviderFeedback extends Feedback {

    /**
     * Это поле не должно заполняться из плагина, оно используется кассой для своих внутренних нужд.<br>
     * Поле заполняется при передаче плагину фидбека на отправку.
     * @since 0.0.46
     */
    private long id;
    /**
     * Количество уже совершенных попыток отправки данной информации в процессинговый центр.
     * <p>
     * NOTE: предполагается, что данное поле будет модифицировать сам плагин лояльности и, на основании достижения количества попыток
     * какого-либо порога, просто примет решение прекратить навсегда попытки отправить этот фидбэк в ПЦ.
     * Например, этот счетчик можно не менять, если произошла ошибка связи/соединения с ПЦ, и увеличивать при возникновении ошибок обработки
     * фидбэка в самом ПЦ.
     */
    private int attemptsCount = 0;

    /**
     * Стратегия сохранения фидбека, следует ли замещать им все старые, просто добавить новый, etc.
     * null соответствует стратегии {@link FeedbackSavingStrategy#REPLACE}.
     * @since 0.0.30
     */
    private FeedbackSavingStrategy savingStrategy = FeedbackSavingStrategy.REPLACE;

    /**
     * Действие, которое следует выполнять с фидбеком в случае, если чек, в рамках которого он образовался, аннулируется.
     * Старое поведение - удаление фидбека. Этому соответствует значение null или {@link ActionOnReceiptCancellation#REMOVE} в этом поле.
     * @since 0.0.33
     */
    private ActionOnReceiptCancellation actionOnReceiptCancellation = ActionOnReceiptCancellation.REMOVE;

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link LoyProviderFeedback}.
     */
    public LoyProviderFeedback() {
        super(null);
    }

    /**
     * Возвращает внутренний кассовый идентификатор фидбека.
     * @return внутренний кассовый идентификатор фидбека или 0, если такого нет.
     * @since 0.0.46
     * @apiNote Этот метод не предназначен для использования из плагинов. Касса использует его для своих нужд.
     */
    public long getId() {
        return id;
    }

    /**
     * Устанавливает внутренний кассовый идентификатор фидбека.
     * @param id внутренний кассовый идентификатор фидбека.
     * @since 0.0.46
     * @apiNote Этот метод не предназначен для использования из плагинов, его использует касса для своих нужд.
     */
    public void setId(long id) {
        this.id = id;
    }

    /**
     * Вернет количество уже совершенных попыток отправки данной информации в процессинговый центр.
     * @return число, что представляет меру "проблемности" (уже выясненной практическим путем) доставки данного feedback'а в процессинговый центр
     */
    public int getAttemptsCount() {
        return attemptsCount;
    }

    public void setAttemptsCount(int attemptsCount) {
        this.attemptsCount = attemptsCount;
    }

    /**
     * Возвращает стратегию сохранения данного фидбека: следует ли заместить им все старые (в рамках чека), просто добавить новый, etc.
     * null соответствует стратегии {@link FeedbackSavingStrategy#REPLACE}.
     * @return стратегия сохранения фидбека или null, что означает выбор стратегии по умолчанию.
     * @since 0.0.30
     */
    public FeedbackSavingStrategy getSavingStrategy() {
        return savingStrategy;
    }

    /**
     * Устанавливает стратегию сохранения данного фидбека.
     * @param savingStrategy стратегия сохранения фидбека
     * @since 0.0.30
     */
    public void setSavingStrategy(FeedbackSavingStrategy savingStrategy) {
        this.savingStrategy = savingStrategy;
    }

    /**
     * Возвращает действие, которое необходимо произвести с фидбеком в случае, если чек, в рамках
     * которого образовался этот фидбек, аннулируется.
     * @return действие, производимое с фидбеком при аннулировании чека. null соответствует {@link ActionOnReceiptCancellation#REMOVE}.
     * @since 0.0.33
     */
    public ActionOnReceiptCancellation getActionOnReceiptCancellation() {
        return actionOnReceiptCancellation;
    }

    /**
     * Устанавливает действие, которое необходимо произвести с фидбеком в случае, если чек, в рамках которого образовался этот фидбек, аннулируется.
     * @param actionOnReceiptCancellation действие, производмо с фидбеком при аннулировании чека. null соответствует {@link ActionOnReceiptCancellation#REMOVE}.
     */
    public void setActionOnReceiptCancellation(ActionOnReceiptCancellation actionOnReceiptCancellation) {
        this.actionOnReceiptCancellation = actionOnReceiptCancellation;
    }

    @Override
    public String toString() {
        return "LoyProviderFeedback{" +
                "attemptsCount=" + attemptsCount +
                ", savingStrategy=" + savingStrategy +
                ", actionOnReceiptCancellation=" + actionOnReceiptCancellation +
                '}';
    }
}
