package ru.crystals.pos.api.plugin;

import ru.crystals.pos.api.ext.loyal.dto.DiscountCancellationResult;
import ru.crystals.pos.api.ext.loyal.dto.LoyaltyResult;
import ru.crystals.pos.api.ext.loyal.dto.PostDiscountResult;
import ru.crystals.pos.api.ext.loyal.dto.PreDiscountResult;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.FiscalizationFeedback;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.LoyProviderFeedback;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.PreFiscalizationFeedback;
import ru.crystals.pos.spi.receipt.Receipt;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Поставщик "услуг лояльности": может дать скидку на чек, добавить купон, рекламный буклет, насчитать бонусы, и проч.
 *
 * @author aperevozchikov
 */
public interface LoyaltyPlugin {

    /**
     * Подсчитает скидки и другие "преференции" лояльности для указанного чека.
     *
     * @param receipt
     *            чек, <em>с уже данными скидками/преференциями другими "поставщиками лояльности"</em>
     * @return {@code null}, если данный "поставщик лояльности" не активен (отключен), либо аргументы невалидны; иначе - вернет результат применения
     * программ лояльности от данного поставщика "услуг лояльности" к данному чеку
     */
    LoyaltyResult doDiscount(Receipt receipt);

    /**
     * Обрабатывает событие "отмена расчета скидок".<br>
     * Событие наступает, например, в случае, если из режима оплат чека переходят обратно в режим добавления товаров (до того, как хотя бы одна оплата будет добавлена)
     * или при аннулировании чека.
     * @param receipt чек, отменить расчет скидок в котором необходимо.
     * @param results результаты расчета скидок по данному чеку
     * @return структуру {@link DiscountCancellationResult}, содержащую результаты отмены расчета скидок в плагине.
     * @since 0.0.26
     */
    default DiscountCancellationResult cancelDiscount(Receipt receipt, LoyaltyResult results) {
        return null;
    }

    /**
     * Оповещение данного поставщика услуг лояльности о факте фискализации чека.
     * <p>
     * Предполагается, что получив это оповещение, поставщик лояльности подготовит данные обратной связи для своего процессингового центра (ПЦ) -
     * чтоб оповестить его (ПЦ) о предоставленных преференциях в чеке.
     *
     * @param receipt       чек, что был фискализирован
     * @param loyResults    преференции, что данный поставщик услуг лояльности предоставил в этом чеке
     * @return {@code null}, если никакого фидбэка для процессинга не требуется
     * @deprecated данный метод больше не вызывается кассой с версии 10.2.71.0, версии Set API 0.0.46, вместо этого метода кассой вызывается
     * метод {@link #onReceiptFiscalized(Receipt, LoyaltyResult)} по завершению фискализации чека. Однако, из соображений обратной совместимости,
     * реализация по умолчанию метода {@link #onReceiptFiscalized(Receipt, LoyaltyResult)} вызывает данный метод.
     */
    @Deprecated
    default LoyProviderFeedback eventReceiptFiscalized(Receipt receipt, LoyaltyResult loyResults) {
        return null;
    }

    /**
     * Данный метод вызывается кассой при завершении фискализации документа. После фискализации чекового документа
     * становятся доступными его QR, в котором содержится информация, достаточная для запроса этого чекового документа
     * у ОФД (Оператор Фискальных Данных).
     * @see Receipt#getQrCode()
     * @param receipt фискализованный чек
     * @param loyaltyResult результаты расчета скидок по данному чеку.
     * @return какие-либо дополнительные результаты плагина или null или пустая структура, если таковых не требуется.
     * @since 0.0.46
     * @apiNote в реализации по умолчанию данный метод вызывает метод {@link #eventReceiptFiscalized(Receipt, LoyaltyResult)}
     * из соображений поддержания обратной совместимости.
     */
    default FiscalizationFeedback onReceiptFiscalized(Receipt receipt, LoyaltyResult loyaltyResult) {
        LoyProviderFeedback legacyFeedback = eventReceiptFiscalized(receipt, loyaltyResult);
        if(legacyFeedback == null) {
            return null;
        }
        FiscalizationFeedback feedback = new FiscalizationFeedback();
        feedback.getFeedbacks().add(legacyFeedback);
        return feedback;
    }
    /**
     * Событие предфискализации.
     * Вызывается непосредственно перед тем, как чек будет фискализирован.
     *
     * @param receipt - чек, который будет фискализирован
     * @param loyResults    преференции, что данный поставщик услуг лояльности предоставил в этом чеке
     * @return {@code null}, если никакого фидбэка для процессинга не требуется
     * */
    default PreFiscalizationFeedback eventBeforeReceiptFiscalized(Receipt receipt, LoyaltyResult loyResults) {
        return null;
    }

    /**
     * Попытка отправить указанный фидбэк в Процессинговый Центр данного поставщика "услуг лояльности".
     * <p>
     * NOTE: данный метод будет вызываться периодически по мере обнаружения фидбэков, что данный поставщик "услуг лояльности" хотел бы передать в свой ПЦ.
     *
     * @param feedback    представление (XML, JSON-строка, base64, и проч.) фидбэка, что надо отправить.
     *                    NOTE: данный фидбэк был сформирован самим данным плагином лояльности: как результат метода {@link #eventReceiptFiscalized(Receipt, LoyaltyResult)}
     *                    или {@link #onSendFeedback(LoyProviderFeedback)}
     * @return
     * <ol>
     * <li> {@code null}, если более не надо пытаться отправить данный фидбэк в ПЦ (Процессинговый Центр) - по любой причине (либо фидбэк успешно доставлен,
     * либо фидбэк признан "битым" и потому не стоит более пытаться его отправить, т.к. ПЦ его все равно не сможет "понять");
     * <li> фидбэк, что надо попробовать отправить взамен указанного в следующий раз (скорей всего,
     * с той же самой {@link LoyProviderFeedback#getPayload() полезной нагрузкой}, но с увеличенным количеством
     * {@link LoyProviderFeedback#getAttemptsCount()} неудачных попыток отправки})
     * </ol>
     * @deprecated с версии API 0.0.46 данный метод больше не вызывается кассой напрямую и следует использовать метод {@link #onSendFeedback(Collection)}.
     */
    @Deprecated
    default LoyProviderFeedback onSendFeedback(LoyProviderFeedback feedback) {
        return null;
    }

    /**
     * Вызывается кассой для обработки плагином ранее созданных им же фидбеков. Тех самых, что плагин лояльности может образовать в методах
     * {@link #eventBeforeReceiptFiscalized(Receipt, LoyaltyResult)} и {@link #eventReceiptFiscalized(Receipt, LoyaltyResult)}. Размер коллекции передаваемой
     * за вызов метода ограничен кассой разумно большим числом. Реализация данного метода по умолчанию просто вызывает {@link #onSendFeedback(LoyProviderFeedback)}
     * в цикле.
     * @param feedback коллекция фидбеков, образованных плагином. Отсортированы по дате в порядке от свежайшего.
     * @return коллекция фидбеков, которые следует пререпроставить на отложенную отправку. Пустая коллекция или null, если плагин справился со всеми фидбеками.
     * При возврате фидбеков здесь слеудует не создавать новые экземпляры {@link LoyProviderFeedback}, а переиспользовать старые, потому
     * что они содержат внутренний идентификатор, по которым касса узнаёт фидбеки.
     * @since 0.0.46
     */
    default Collection<LoyProviderFeedback> onSendFeedback(Collection<LoyProviderFeedback> feedback) {
        List<LoyProviderFeedback> unsuccessfulFeedbacks = new ArrayList<>();
        for(LoyProviderFeedback f : feedback) {
            LoyProviderFeedback result = onSendFeedback(f);
            if(result != null) {
                unsuccessfulFeedbacks.add(f);
            }
        }
        return unsuccessfulFeedbacks;
    }

    /**
     * Данный метод вызывается кассой после завершения расчета скидок.<br>
     * Некоторым системам лояльности важно допускать оплачивание бонусами не более величины Х от чека. Например, допускать оплачивать бонусами СУММА_ЧЕКА - 1 рубль.
     * Однако, после того как скидки были посчитаны в плагине, иные скидки могли быть добавлены в ином плагине, а также кассой. Тогда становится возможной ситуация,
     * когда доступная величина списания стенет большей, чем сумма чека и условие "допустить списание не более, чем СУММА_ЧЕКА - 1 рубль" не будет выполнено. Этот метод
     * предоставляет возможность исправить такую ситуацию, подкорректировав доступную величну к списанию бонусов.
     * @param receipt чек с результирующим расчетом скидок
     * @return результат корректировки расчета скидок плагином лояльности или null, если плагин не выполняет данную функцию.
     * @since 0.0.39
     */
    default PostDiscountResult onDiscountCalculationFinished(Receipt receipt) {
        return null;
    }

    /**
     * Данный метод вызывается кассой перед началом расчета скидок.
     *
     * @param originalReceipt представление оригинального чека
     * @since 0.0.70
     */
    default PreDiscountResult onDiscountCalculationStarted(Receipt originalReceipt) {
        return null;
    }
}
