package ru.crystals.pos.api.plugin.payment;

import ru.crystals.pos.api.plugin.TransactionalRefundPlugin;
import ru.crystals.pos.spi.plugin.payment.PaymentToRefund;
import ru.crystals.pos.spi.plugin.payment.RefundPreparationRequest;
import ru.crystals.pos.spi.plugin.payment.TransactionalRefundRequest;
import ru.crystals.pos.spi.receipt.Receipt;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

/**
 * "Замена" для одной или нескольких транзакций, доступных для возврата
 *
 * @since 1.5.0
 */
public class PaymentToRefundReplacement {

    /**
     * Список транзакций, для которых применяется эта замена.
     * <p>
     * Касса ожидает в списке те же самые экземпляры {@link PaymentToRefund}, что были переданы
     * в {@link RefundPreparationRequest#getPaymentsToRefund()} (сравнивает по <code>==</code> ).
     * <p>
     * Если список пустой, замена считается недействительной.
     * <p>
     * Если в списке встречается транзакция из других замен результата подготовки возврата ({@link RefundPreparationResult#getPaymentReplacements()}),
     * то все замены с этой транзакцией считаются недействительными.
     * <hr>
     * <h3>Объединение транзакций для возврата</h3>
     * <p>
     * Если в списке несколько транзакций, то эти транзакции объединяются и при возврате будут переданы в плагин в виде одного запроса на возврат
     * со ссылкой на список всех транзакций ({@link TransactionalRefundRequest#getPaymentsToRefund()}).
     * <p>
     * Для поддержки возврата нескольких необходимо для возвратов по чеку реализовать {@link TransactionalRefundPlugin#doTransactionalRefund(TransactionalRefundRequest)}.
     * <p>
     * Рекомендуется использовать эту возможность только для случаев, когда можно вернуть несколько транзакций одним обращением в процессинг.
     * <p>
     * При объединении нескольких транзакций общая сумма транзакций может превысить сумму текущего чека возврата, в этом случае касса уменьшит общую сумму перед
     * передачей в {@link TransactionalRefundRequest} (за счет уменьшения суммы отдельных транзакций или их исключения из списка для возврата).
     * Если требуется точное соответствие списка объединенных транзакций, плагин должен проконтролировать, что сумма объединенных транзакций не превышает общей суммы,
     * оставшейся для возврата в этом чеке (см. {@link RefundPreparationRequest#getRefundReceipt()} и {@link Receipt#getSurchargeSum()})
     */
    private List<PaymentToRefund> usedFor = Collections.emptyList();

    /**
     * Активирует возможность вернуть транзакцию дополнительными способами,
     * заданными общей настройкой "Способы возврата, дополнительно доступные по решению плагина"
     * <p>
     * Признак не ограничивает возможность возврата способами из общей настройки "Дополнительные способы возврата, доступные всегда"
     */
    private boolean useAlternativePaymentTypesToRefund;

    /**
     * Запрещает возврат транзакции через плагин.
     * <p>
     * Можно будет вернуть транзакцию только дополнительными способами.
     * Если дополнительные способы не настроены, возврат чека с такой транзакцией может быть невозможен
     */
    private boolean restrictRefundByPlugin;

    /**
     * Название, которое должно отображаться для транзакции в списке транзакций, доступных для возврата
     * <p>
     * Если не заполнено, используется название типа оплаты из метаинформации плагина
     */
    private String description;

    /**
     * Часть суммы исходной транзакции/транзакций {@link #usedFor} которая должна быть обязательно возвращена этим плагином (или его заменами) в этом чеке возврата.
     * <p>
     * Используется, когда плагином можно оплачивать только определенные товары, чтобы возврат суммы этих товаров проходил также через плагин.
     * <p>
     * Если значение равно {@code null}, возможен возврат любой части исходной транзакции/транзакций без ограничений
     * <p>
     * Если значение равно нулю, то возврат исходной транзакции в этом чеке возврата недоступен.
     * <p>
     * Касса игнорирует это значение или всю замену с таким значением в следующих случаях
     * <ul>
     *     <li>плагин не реализует {@link TransactionalRefundPlugin#doTransactionalRefund(TransactionalRefundRequest)}</li>
     *     <li>значение превышает общую доступную сумму исходной транзакции/транзакций для возврата в этом чеке ({@link PaymentToRefund#getSumToRefund()})</li>
     *     <li>значение меньше ({@link PaymentToRefund#getSumToRefund()}) настолько, что остальными оплатами в чеке невозможно покрыть эту разницу)</li>
     *     <li>значение меньше нуля</li>
     * </ul>
     */
    private BigDecimal requiredRefundableSum;

    public PaymentToRefundReplacement() {

    }

    private PaymentToRefundReplacement(List<PaymentToRefund> usedFor, boolean useAlternativePaymentTypesToRefund, boolean restrictRefundByPlugin, String description) {
        this.usedFor = Objects.requireNonNull(usedFor);
        this.useAlternativePaymentTypesToRefund = useAlternativePaymentTypesToRefund;
        this.restrictRefundByPlugin = restrictRefundByPlugin;
        this.description = description;
    }

    public static PaymentToRefundReplacement withDescription(PaymentToRefund usedFor, String description) {
        final PaymentToRefundReplacement replacement = new PaymentToRefundReplacement();
        replacement.setUsedFor(Collections.singletonList(Objects.requireNonNull(usedFor)));
        replacement.setDescription(Objects.requireNonNull(description));
        return replacement;
    }

    public static PaymentToRefundReplacement forPayments(List<PaymentToRefund> usedFor) {
        final PaymentToRefundReplacement replacement = new PaymentToRefundReplacement();
        replacement.setUsedFor(usedFor);
        return replacement;
    }

    public List<PaymentToRefund> getUsedFor() {
        return usedFor;
    }

    public void setUsedFor(List<PaymentToRefund> usedFor) {
        this.usedFor = Objects.requireNonNull(usedFor);
    }

    public boolean isUseAlternativePaymentTypesToRefund() {
        return useAlternativePaymentTypesToRefund;
    }

    public void setUseAlternativePaymentTypesToRefund(boolean useAlternativePaymentTypesToRefund) {
        this.useAlternativePaymentTypesToRefund = useAlternativePaymentTypesToRefund;
    }

    public boolean isRestrictRefundByPlugin() {
        return restrictRefundByPlugin;
    }

    public void setRestrictRefundByPlugin(boolean restrictRefundByPlugin) {
        this.restrictRefundByPlugin = restrictRefundByPlugin;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public BigDecimal getRequiredRefundableSum() {
        return requiredRefundableSum;
    }

    public void setRequiredRefundableSum(BigDecimal requiredRefundableSum) {
        this.requiredRefundableSum = requiredRefundableSum;
    }

    @Override
    public String toString() {
        return "PaymentToRefundReplacement{" +
               "usedFor=" + usedFor +
               ", useAlternativePaymentTypesToRefund=" + useAlternativePaymentTypesToRefund +
               ", restrictRefundByPlugin=" + restrictRefundByPlugin +
               ", description='" + description + '\'' +
               ", requiredRefundableSum='" + requiredRefundableSum + '\'' +
               '}';
    }
}
